#!/bin/bash

SYSREPORT_NAME=sysreport_$(hostname)_$(date +%Y%m%d_%H%M%S)
ARCHIVE=velostrata_${SYSREPORT_NAME}.tar.gz
STAGE_DIR=/tmp/velostrata/${SYSREPORT_NAME}
TAR_ROOT=sysreport

function create_dir {
	mkdir -p ${STAGE_DIR}/${TAR_ROOT}
	mkdir ${STAGE_DIR}/${TAR_ROOT}/bootcfg
}

function capture_output {
	local cmd outfile temp_file
	cmd=$1
	outfile=$2

	temp_file="/tmp/`basename ${outfile}`.$$.tmp"
	${cmd} > ${temp_file}
	mv ${temp_file} ${outfile}
}

function add_syslog {
	cp /var/log/messages* ${STAGE_DIR}/${TAR_ROOT}
	cp /var/log/syslog* ${STAGE_DIR}/${TAR_ROOT}
	cp /var/log/boot.msg ${STAGE_DIR}/${TAR_ROOT}
}

function add_history {
	capture_output 'history' ${STAGE_DIR}/${TAR_ROOT}/history_out
}

function add_iscsi_devs_timeout {
	file_name="/tmp/iscsi_devs_timeout.$$.tmp"

	echo "iSCSI devices timeout:" >> ${file_name}

	for dev in `iscsiadm -m session -P 3 | grep "Attached scsi disk" | awk '{print $4}'`; do
		timeout=`cat /sys/block/${dev}/device/timeout`

		echo "${dev}: timeout = ${timeout}" >> ${file_name}
	done
	mv ${file_name} ${STAGE_DIR}/${TAR_ROOT}/iscsi_devs_timeout
}

function add_ps {
	capture_output 'ps aux' ${STAGE_DIR}/${TAR_ROOT}/ps_out
}

function add_lsmod {
	capture_output 'lsmod' ${STAGE_DIR}/${TAR_ROOT}/lsmod_out
}

function add_os_version_info {
	capture_output 'uname' ${STAGE_DIR}/${TAR_ROOT}/uname_out
	cp /etc/issue ${STAGE_DIR}/${TAR_ROOT}
}

function add_disk_info {
	capture_output 'df -ah' ${STAGE_DIR}/${TAR_ROOT}/df_out
	capture_output 'mount' ${STAGE_DIR}/${TAR_ROOT}/mount_out
	capture_output 'lsblk' ${STAGE_DIR}/${TAR_ROOT}/lsblk_out
	capture_output 'blkid' ${STAGE_DIR}/${TAR_ROOT}/blkid_out
	capture_output 'fdisk -l' ${STAGE_DIR}/${TAR_ROOT}/fdisk_out
	capture_output 'parted -l' ${STAGE_DIR}/${TAR_ROOT}/parted_out
	capture_output 'cat /proc/partitions' ${STAGE_DIR}/${TAR_ROOT}/partitions_out
	capture_output 'ls -l /dev/disk/by-id' ${STAGE_DIR}/${TAR_ROOT}/disks_by_id_out
	capture_output 'ls -l /dev/disk/by-uuid' ${STAGE_DIR}/${TAR_ROOT}/disks_by_uuid_out
}

function add_iscsiadm {
	capture_output 'iscsiadm -m node' ${STAGE_DIR}/${TAR_ROOT}/iscsiadm_node_out
	capture_output 'iscsiadm -m session -P 3' ${STAGE_DIR}/${TAR_ROOT}/iscsiadm_session_out
}

function add_sg_map {
	capture_output 'sg_map -i -x' ${STAGE_DIR}/${TAR_ROOT}/sg_map_out
}

function add_crontab {
	capture_output 'crontab -l' ${STAGE_DIR}/${TAR_ROOT}/crontab_out
}

function add_network_configuration {
	local out_dir

	outdir=${STAGE_DIR}/${TAR_ROOT}/network
	${outdir}

	capture_output 'ifconfig -a' ${outdir}/ifconfig_out
	capture_output 'ip addr show' ${outdir}/ip_addr_out
	capture_output 'ip link show' ${outdir}/ip_link_out
	cat /etc/network/interfaces > ${outdir}/ub-network-interfaces

	cat /etc/resolv.conf > ${outdir}/resolv.conf

	for ff in `ls /etc/sysconfig/network-scripts/ifcfg-*` ; do
		cat ${ff} > ${outdir}/$(basename ${ff})
	done

	for ff in `ls /etc/sysconfig/network/ifcfg-*` ; do
		cat ${ff} > ${outdir}/$(basename ${ff})
	done
}

function add_date {
	capture_output 'date' ${STAGE_DIR}/${TAR_ROOT}/date_out
}

function add_velostrata_logs {
	cp /var/tmp/velostrata_online_prep.log ${STAGE_DIR}/${TAR_ROOT}
	cp /var/tmp/velostrata_trim.log ${STAGE_DIR}/${TAR_ROOT}
}

function add_installed_packages {
	capture_output 'rpm -qa' ${STAGE_DIR}/${TAR_ROOT}/rpms_out
	capture_output 'dpkg -l' ${STAGE_DIR}/${TAR_ROOT}/dpkg_out
}

function add_cfg {
	cp /etc/iscsi/* ${STAGE_DIR}/${TAR_ROOT}
	cp /etc/multipath/* ${STAGE_DIR}/${TAR_ROOT}
	cp /etc/fstab ${STAGE_DIR}/${TAR_ROOT}
	cp /etc/hosts ${STAGE_DIR}/${TAR_ROOT}
	cp /etc/lvm/lvm.conf ${STAGE_DIR}/${TAR_ROOT}
	cp /boot/grub/grub.conf ${STAGE_DIR}/${TAR_ROOT}
	cp /boot/grub/menu.lst ${STAGE_DIR}/${TAR_ROOT}
}

function add_multipath {
	capture_output 'multipath -ll' ${STAGE_DIR}/${TAR_ROOT}/multipath_ll_out
	capture_output 'multipath -v6' ${STAGE_DIR}/${TAR_ROOT}/multipath_out
}

function add_ibft_entries {
	file_name="/tmp/ibft_out.$$.tmp"

	if [ -d /sys/firmware/ibft/ ]; then
		for f in `find /sys/firmware/ibft/target* -type f`; do
			val=`cat ${f}`
			echo "${f}: ${val}" >> ${file_name}
		done
	
		mv ${file_name} ${STAGE_DIR}/${TAR_ROOT}/ibft_out
	fi
}

function add_mbr {
	file_name="/tmp/bootable_devices_mbr.$$.tmp"

	if which xxd &> /dev/null; then
		hex_viewer=xxd
	else
		hex_viewer=hexdump
	fi

	bootable_devices=`fdisk -l | grep '^/dev/[a-z]*[0-9]' | awk '$2 == "*" { print $1 }' | sed 's/[0-9]*//g' | sort -u`
	for dev in ${bootable_devices}; do
		echo ${dev} >> ${file_name}
		dd if=${dev} bs=512 count=1 2> /dev/null | ${hex_viewer} >> ${file_name}
	done

	mv ${file_name} ${STAGE_DIR}/${TAR_ROOT}/bootable_devices_mbr.out
}

function add_boot_config {
        for grubfile in /boot/grub/menu.lst /boot/grub/grub.conf /boot/grub/grub.cfg /etc/default/grub /boot/grub2/grub.cfg ; do
                cp ${grubfile} ${STAGE_DIR}/${TAR_ROOT}/bootcfg/
        done

        cp /etc/default/grub ${STAGE_DIR}/${TAR_ROOT}/bootcfg/default_grub
        grubby --default-kernel --bad-image-okay > ${STAGE_DIR}/${TAR_ROOT}/bootcfg/default_kernel
}

function add_initramfs_content {
        for init_file in /boot/initramfs-*.img ; do
                lsinitrd $init_file >> ${STAGE_DIR}/${TAR_ROOT}/lsinitrd.out ;
        done
}

function pack
{
	tar -czf ${ARCHIVE} -C ${STAGE_DIR} ${TAR_ROOT}
	rm -rf ${STAGE_DIR}

	OUT=SYSREPORT=$(readlink -f ${ARCHIVE})
	echo ${OUT}
}

create_dir
add_syslog
add_history
add_iscsi_devs_timeout
add_ps
add_lsmod
add_os_version_info
add_disk_info
add_iscsiadm
add_sg_map
add_multipath
add_crontab
add_network_configuration
add_date
add_velostrata_logs
add_installed_packages
add_cfg
add_ibft_entries
add_mbr
add_boot_config
add_initramfs_content
pack
