#!/bin/bash

source /opt/velostrata/velostrata-functions

LOG=/etc/velostrata/gcp_agent_env.log
NTPCONF=/etc/ntp.conf
echo `date` >> $LOG


function run_with_log() {
	$* 2>&1 | tee -a $LOG
}

function configure_ntp() {
	cp -f ${NTPCONF} /etc/velostrata/backup/ntp.conf.pre-gcp
	sed -i 's/^pool /#pool /g' ${NTPCONF}
	if ! grep -q metadata.google.internal ${NTPCONF} ; then
		sed -i '0,/^server /s/^server /server metadata.google.internal\n&/' ${NTPCONF}
	fi
	if ! grep -q metadata.google.internal ${NTPCONF} ; then
		echo "server metadata.google.internal" >> ${NTPCONF}
	fi

	enable_and_run_service ntp
	enable_and_run_service ntpd
}

function yum_add_google_repo() {
	local dist
	dist=$1

	# add EPEL repos
	if [ ${dist} -gt 6 ] ; then
		run_with_log rpm --import http://dl.fedoraproject.org/pub/epel/RPM-GPG-KEY-EPEL-${dist}
		cat > /etc/yum.repos.d/epel.repo <<EOF
[epel]
name=Extra Packages for Enterprise Linux ${dist} - \$basearch
#baseurl=http://download.fedoraproject.org/pub/epel/${dist}/\$basearch
metalink=https://mirrors.fedoraproject.org/metalink?repo=epel-${dist}&arch=\$basearch
failovermethod=priority
enabled=1
gpgcheck=1
gpgkey=file:///etc/pki/rpm-gpg/RPM-GPG-KEY-EPEL-${dist}

[epel-debuginfo]
name=Extra Packages for Enterprise Linux ${dist} - \$basearch - Debug
#baseurl=http://download.fedoraproject.org/pub/epel/${dist}/\$basearch/debug
metalink=https://mirrors.fedoraproject.org/metalink?repo=epel-debug-${dist}&arch=\$basearch
failovermethod=priority
enabled=0
gpgkey=file:///etc/pki/rpm-gpg/RPM-GPG-KEY-EPEL-${dist}
gpgcheck=1

[epel-source]
name=Extra Packages for Enterprise Linux ${dist} - \$basearch - Source
#baseurl=http://download.fedoraproject.org/pub/epel/${dist}/SRPMS
metalink=https://mirrors.fedoraproject.org/metalink?repo=epel-source-${dist}&arch=\$basearch
failovermethod=priority
enabled=0
gpgkey=file:///etc/pki/rpm-gpg/RPM-GPG-KEY-EPEL-${dist}
gpgcheck=1
EOF
	fi

	# add GCP repos
	cat > /etc/yum.repos.d/google-cloud.repo <<EOF
[google-cloud-compute]
name=Google Cloud Compute
baseurl=https://packages.cloud.google.com/yum/repos/google-cloud-compute-el${dist}-x86_64
enabled=1
gpgcheck=1
repo_gpgcheck=1
gpgkey=https://packages.cloud.google.com/yum/doc/yum-key.gpg
       https://packages.cloud.google.com/yum/doc/rpm-package-key.gpg
EOF
}

function apt_add_google_repo() {
	curl https://packages.cloud.google.com/apt/doc/apt-key.gpg | apt-key add -

	# only relevant for google-cloud-sdk
	export CLOUD_SDK_REPO="cloud-sdk-$(lsb_release -c -s)"
	echo "deb http://packages.cloud.google.com/apt $CLOUD_SDK_REPO main" | tee -a /etc/apt/sources.list.d/google-cloud-sdk.list

	cat > /etc/apt/sources.list.d/google-cloud.list <<EOF
deb http://packages.cloud.google.com/apt google-compute-engine-stretch-stable main
deb http://packages.cloud.google.com/apt google-cloud-packages-archive-keyring-stretch main
EOF

	run_with_log apt-get update
}

function suse_add_google_repos() {
    local ver sp license_type
	ver=$(get_suse_version)
	sp=$(get_suse_sp)
    license_type="$(get_gcp_license_type)"
	run_with_log echo Detected SUSE${ver} SP${sp}
	run_with_log echo License type: ${license_type}

	if [[ "${ver}" == "11" ]] ; then
		run_with_log zypper modifyrepo -e nu_novell_com:SLE11-Public-Cloud-Module
	else
		run_with_log SUSEConnect -p sle-module-public-cloud/${ver}/x86_64
	fi
	run_with_log zypper -n refs
	run_with_log zypper -n refresh

	if [ "${license_type}" == "gcp" ]; then
	    suse_activate_gcp_license "${ver}" "${sp}"
	fi
}

function suse_activate_gcp_license() {
    local ver sp
	ver="$1"
	sp="$2"

	run_with_log zypper install -y cloud-regionsrv-client cloud-regionsrv-client-plugin-gce regionServiceClientConfigGCE regionServiceCertsGCE

	if [[ "${ver}" == "11" ]] ; then
		/usr/sbin/registercloudguest
	elif [[ "${ver}" == "12" ]] ; then
		enable_and_run_service google-clock-skew-daemon.service
		enable_and_run_service google-instance-setup.service
		enable_and_run_service google-ip-forwarding-daemon.service
		enable_and_run_service google-shutdown-scripts.service
		enable_and_run_service google-startup-scripts.service
		enable_and_run_service guestregister.service
		enable_and_run_service haveged.service
	fi

	run_with_log zypper -n refs
	run_with_log zypper -n refresh
}


function rpm_remove()
{
	local agent
	agent=$1

	run_with_log rpm -e `rpm -qa | grep ${agent}`
}

function enable_and_run_service()
{
	local svc
	svc=$1

	if which systemctl >/dev/null ; then
		run_with_log systemctl enable ${svc}
		run_with_log systemctl restart ${svc}
	elif which update-rc.d >/dev/null ; then
		run_with_log update-rc.d ${svc} enable
		run_with_log service ${svc} restart
	elif which chkconfig >/dev/null ; then
		run_with_log chkconfig ${svc} on
		run_with_log service ${svc} restart
	fi
}

function disable_service()
{
	local svc
	svc=$1

	if which systemctl >/dev/null ; then
		run_with_log systemctl disable ${svc}
	elif which update-rc.d >/dev/null ; then
		run_with_log update-rc.d ${svc} disable
	elif which chkconfig >/dev/null ; then
		run_with_log chkconfig ${svc} off
	fi
}

# Adding in GCE RHUI package
function install_gce_rhui ()
{
    local license_type dist
    license_type="$(get_gcp_license_type)"
	if is_rhel6_family ; then
		dist=6
	else
		dist=7
	fi
    run_with_log echo Detected RHEL${dist}
    run_with_log echo License type: ${license_type}

    yum_add_google_repo ${dist}

    if [ "${license_type}" == "gcp" ]; then
        run_with_log yum remove -y *rhui*
        run_with_log yum install -y google-rhui-client-rhel${dist}
    fi
}

# 2.1. NTP configuration points to the NTP server, “metadata.google.internal”.
# 3.1. Compute Engine Linux Guest Environment is installed from packages or provided repositories.

if is_ubuntu_family ; then
	run_with_log dpkg -r amazon-ssm-agent
	run_with_log dpkg -r awscli
	disable_service walinuxagent

	run_with_log apt-get update
	run_with_log apt-get install -y ntp
	configure_ntp

	apt_add_google_repo
	run_with_log apt-get install -y google-cloud-packages-archive-keyring
	run_with_log apt-get update

	run_with_log apt-get install -y google-cloud-sdk
	run_with_log apt-get install -y python-google-compute-engine python3-google-compute-engine
	run_with_log apt-get install -y google-compute-engine-oslogin
else
	rpm_remove amazon-ssm-agent
	rpm_remove awscli
	rpm_remove aws-cli
	rpm_remove rh-amazon-rhui-client
	disable_service waagent

	if is_rhel_family ; then
		install_gce_rhui

		run_with_log yum install -y ntp
		configure_ntp

		# update python first - workaround for HA-6002
		run_with_log yum install -y python
		run_with_log yum install -y google-compute-engine python-google-compute-engine
	elif is_suse ; then
		suse_add_google_repos

		run_with_log zypper -n install ntp
		configure_ntp

		run_with_log zypper -n install google-compute-engine-init
		run_with_log zypper -n install google-cloud-sdk
		run_with_log zypper -n install python-google-apputils
	fi
fi

touch /etc/velostrata/actions/bypass/gcp-agent-env
exit 0
